package java.security.cert;

import sun.security.x509.CRLNumberExtension;
import sun.security.util.DerInputStream;
import java.util.Collections;
import sun.security.x509.X500Name;
import java.io.IOException;
import java.util.Iterator;
import java.util.Collection;
import java.util.Date;
import java.math.BigInteger;
import javax.security.auth.x500.X500Principal;
import java.util.HashSet;
import sun.security.util.Debug;

public class X509CRLSelector implements CRLSelector
{
    private static final Debug debug;
    private HashSet<Object> issuerNames;
    private HashSet<X500Principal> issuerX500Principals;
    private BigInteger minCRL;
    private BigInteger maxCRL;
    private Date dateAndTime;
    private X509Certificate certChecking;
    private long skew;
    
    public X509CRLSelector() {
        this.skew = 0L;
    }
    
    public void setIssuers(Collection<X500Principal> c) {
        if (c == null || c.isEmpty()) {
            this.issuerNames = null;
            this.issuerX500Principals = null;
        }
        else {
            this.issuerX500Principals = new HashSet(c);
            this.issuerNames = new HashSet();
            Iterator<X500Principal> iterator = this.issuerX500Principals.iterator();
            while (iterator.hasNext()) {
                this.issuerNames.add(((X500Principal)iterator.next()).getEncoded());
            }
        }
    }
    
    public void setIssuerNames(Collection<?> collection) throws IOException {
        if (collection == null || collection.size() == 0) {
            this.issuerNames = null;
            this.issuerX500Principals = null;
        }
        else {
            HashSet<Object> cloneAndCheckIssuerNames = cloneAndCheckIssuerNames(collection);
            this.issuerX500Principals = parseIssuerNames(cloneAndCheckIssuerNames);
            this.issuerNames = cloneAndCheckIssuerNames;
        }
    }
    
    public void addIssuer(X500Principal x500Principal) {
        this.addIssuerNameInternal(x500Principal.getEncoded(), x500Principal);
    }
    
    public void addIssuerName(String dname) throws IOException {
        this.addIssuerNameInternal(dname, new X500Name(dname).asX500Principal());
    }
    
    public void addIssuerName(byte[] name) throws IOException {
        this.addIssuerNameInternal(name.clone(), new X500Name(name).asX500Principal());
    }
    
    private void addIssuerNameInternal(Object e, X500Principal e2) {
        if (this.issuerNames == null) {
            this.issuerNames = new HashSet();
        }
        if (this.issuerX500Principals == null) {
            this.issuerX500Principals = new HashSet();
        }
        this.issuerNames.add(e);
        this.issuerX500Principals.add(e2);
    }
    
    private static HashSet<Object> cloneAndCheckIssuerNames(Collection<?> collection) throws IOException {
        HashSet<byte[]> set = new HashSet();
        for (Object next : collection) {
            if (!(next instanceof byte[]) && !(next instanceof String)) {
                throw new IOException("name not byte array or String");
            }
            if ((next instanceof byte[])) {
                set.add(((byte[])(byte[])next).clone());
            }
            else {
                set.add(next);
            }
        }
        return set;
    }
    
    private static HashSet<Object> cloneIssuerNames(Collection<Object> collection) {
        try {
            return cloneAndCheckIssuerNames(collection);
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
    }
    
    private static HashSet<X500Principal> parseIssuerNames(Collection<Object> collection) throws IOException {
        HashSet<X500Principal> set = new HashSet();
        for (Object next : collection) {
            if ((next instanceof String)) {
                set.add(new X500Name((String)next).asX500Principal());
            }
            else {
                try {
                    set.add(new X500Principal((byte[])(byte[])next));
                }
                catch (IllegalArgumentException cause) {
                    throw (IOException)new IOException("Invalid name").initCause(cause);
                }
            }
        }
        return set;
    }
    
    public void setMinCRLNumber(BigInteger minCRL) {
        this.minCRL = minCRL;
    }
    
    public void setMaxCRLNumber(BigInteger maxCRL) {
        this.maxCRL = maxCRL;
    }
    
    public void setDateAndTime(Date date) {
        if (date == null) {
            this.dateAndTime = null;
        }
        else {
            this.dateAndTime = new Date(date.getTime());
        }
        this.skew = 0L;
    }
    
    void setDateAndTime(Date date, long skew) {
        this.dateAndTime = ((date == null) ? null : new Date(date.getTime()));
        this.skew = skew;
    }
    
    public void setCertificateChecking(X509Certificate certChecking) {
        this.certChecking = certChecking;
    }
    
    public Collection<X500Principal> getIssuers() {
        if (this.issuerX500Principals == null) {
            return null;
        }
        return Collections.unmodifiableCollection(this.issuerX500Principals);
    }
    
    public Collection<Object> getIssuerNames() {
        if (this.issuerNames == null) {
            return null;
        }
        return cloneIssuerNames(this.issuerNames);
    }
    
    public BigInteger getMinCRL() {
        return this.minCRL;
    }
    
    public BigInteger getMaxCRL() {
        return this.maxCRL;
    }
    
    public Date getDateAndTime() {
        if (this.dateAndTime == null) {
            return null;
        }
        return (Date)this.dateAndTime.clone();
    }
    
    public X509Certificate getCertificateChecking() {
        return this.certChecking;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("X509CRLSelector: [\n");
        if (this.issuerNames != null) {
            sb.append("  IssuerNames:\n");
            Iterator<Object> iterator = this.issuerNames.iterator();
            while (iterator.hasNext()) {
                sb.append("    " + iterator.next() + "\n");
            }
        }
        if (this.minCRL != null) {
            sb.append("  minCRLNumber: " + this.minCRL + "\n");
        }
        if (this.maxCRL != null) {
            sb.append("  maxCRLNumber: " + this.maxCRL + "\n");
        }
        if (this.dateAndTime != null) {
            sb.append("  dateAndTime: " + this.dateAndTime + "\n");
        }
        if (this.certChecking != null) {
            sb.append("  Certificate being checked: " + this.certChecking + "\n");
        }
        sb.append("]");
        return sb.toString();
    }
    
    @Override
    public boolean match(CRL crl) {
        if (!(crl instanceof X509CRL)) {
            return false;
        }
        X509CRL x509CRL = (X509CRL)crl;
        if (this.issuerNames != null) {
            X500Principal issuerX500Principal = x509CRL.getIssuerX500Principal();
            Iterator<X500Principal> iterator = this.issuerX500Principals.iterator();
            int n = 0;
            while (n == 0 && iterator.hasNext()) {
                if (!((X500Principal)iterator.next()).equals(issuerX500Principal)) {
                    continue;
                }
                n = 1;
            }
            if (n == 0) {
                if (X509CRLSelector.debug != null) {
                    X509CRLSelector.debug.println("X509CRLSelector.match: issuer DNs don't match");
                }
                return false;
            }
        }
        if (this.minCRL != null || this.maxCRL != null) {
            byte[] extensionValue = x509CRL.getExtensionValue("2.5.29.20");
            if (extensionValue == null && X509CRLSelector.debug != null) {
                X509CRLSelector.debug.println("X509CRLSelector.match: no CRLNumber");
            }
            BigInteger value;
            try {
                value = new CRLNumberExtension(Boolean.FALSE, new DerInputStream(extensionValue).getOctetString()).get("value");
            }
            catch (IOException ex) {
                if (X509CRLSelector.debug != null) {
                    X509CRLSelector.debug.println("X509CRLSelector.match: exception in decoding CRL number");
                }
                return false;
            }
            if (this.minCRL != null && value.compareTo(this.minCRL) < 0) {
                if (X509CRLSelector.debug != null) {
                    X509CRLSelector.debug.println("X509CRLSelector.match: CRLNumber too small");
                }
                return false;
            }
            else if (this.maxCRL != null && value.compareTo(this.maxCRL) > 0) {
                if (X509CRLSelector.debug != null) {
                    X509CRLSelector.debug.println("X509CRLSelector.match: CRLNumber too large");
                }
                return false;
            }
        }
        if (this.dateAndTime != null) {
            Date thisUpdate = x509CRL.getThisUpdate();
            Date nextUpdate = x509CRL.getNextUpdate();
            if (nextUpdate == null) {
                if (X509CRLSelector.debug != null) {
                    X509CRLSelector.debug.println("X509CRLSelector.match: nextUpdate null");
                }
                return false;
            }
            else {
                Date dateAndTime = this.dateAndTime;
                Date dateAndTime2 = this.dateAndTime;
                if (this.skew > 0L) {
                    dateAndTime = new Date(this.dateAndTime.getTime() + this.skew);
                    dateAndTime2 = new Date(this.dateAndTime.getTime() - this.skew);
                }
                if (dateAndTime2.after(nextUpdate) || dateAndTime.before(thisUpdate)) {
                    if (X509CRLSelector.debug != null) {
                        X509CRLSelector.debug.println("X509CRLSelector.match: update out-of-range");
                    }
                    return false;
                }
            }
        }
        return true;
    }
    
    @Override
    public Object clone() {
        try {
            X509CRLSelector x509CRLSelector = (X509CRLSelector)super.clone();
            if (this.issuerNames != null) {
                x509CRLSelector.issuerNames = new HashSet(this.issuerNames);
                x509CRLSelector.issuerX500Principals = new HashSet(this.issuerX500Principals);
                return x509CRLSelector;
            }
            return x509CRLSelector;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    static {
        CertPathHelperImpl.initialize();
        debug = Debug.getInstance("certpath");
    }
}
